#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-Sun (@Fred-Sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_publicipprefix

version_added: "2.2.0"

short_description: Manage Azure Public IP prefix

description:
    - Create, update and delete a Public IP prefix.

options:
    resource_group:
        description:
            - Name of resource group with which the Public IP prefix is associated.
        required: true
        type: str
    name:
        description:
            - Name of the Public IP prefix.
        required: true
        type: str
    state:
        description:
            - Assert the state of the Public IP. Use C(present) to create or update a and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present
    location:
        description:
            - Valid Azure location. Defaults to location of the resource group.
        type: str
    sku:
        description:
            - The public IP prefix SKU.
        type: dict
        suboptions:
            name:
                description:
                    - Name of a public IP prefix SKU.
                type: str
                choices:
                    - Standard
            tier:
                description:
                    - Tier of a public IP prefix SKU.
                type: str
                choices:
                    - Regional
                    - Global
    custom_ip_prefix:
        description:
            - The Custom IP prefix that this prefix is associated with.
        type: dict
        suboptions:
            id:
                description:
                    - Resource ID.
                type: str
    extended_location:
        description:
            - The extended location of the public ip address.
        type: str
    ip_tags:
        description:
            - The list of tags associated with the public IP prefix.
        type: list
        elements: dict
        suboptions:
            ip_tag_type:
                description:
                    - The IP tag type. Example as FirstPartyUsage.
                type: str
            tag:
                description:
                    - The value of the IP tag associated with the public IP. Example as SQL.
                type: str
    public_ip_address_version:
        description:
            - The public IP address version.
        type: str
        choices:
            - IPV4
            - IPV6
    zones:
        description:
            - A list of availability zones denoting the IP prefix allocated for the resource needs to come from.
        type: list
        elements: str
        choices:
            - '1'
            - '2'
            - '3'
    prefix_length:
        description:
            - The Length of the Public IP Prefix.
        type: int

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Create a public ip prefix
  azure_rm_publicipprefix:
    resource_group: myResourceGroup
    name: my_public_ip
    public_ip_address_version: IPV4
    prefix_length: 29
    sku:
      name: Standard
      tier: Regional
    zones:
      - 1
    tags:
      key1: value1

- name: Delete public ip prefix
  azure_rm_publicipprefix:
    resource_group: myResourceGroup
    name: my_public_ipprefix
    state: absent
'''

RETURN = '''
state:
    description:
        - List of public IP prefixes dicts.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: /subscriptions/xxx---xxxxx/resourceGroups/v-xisuRG/providers/Microsoft.Network/publicIPPrefixes/pipb57dc95224
        name:
            description:
                - Name of the public IP prefix.
            returned: always
            type: str
            sample: prefix57dc95224
        type:
            description:
                - Resource type.
            returned: always
            type: str
            sample: "Microsoft.Network/publicIPPrefixes"
        location:
            description:
                - Resource location.
            returned: always
            type: str
            sample: eastus
        tags:
            description:
                - Resource tags.
            returned: always
            type: dict
            sample: {
                    "delete": "on-exit",
                    "testing": "testing"
                    }
        public_ip_address_version:
            description:
                - The public IP address version.
                - Possible values are C(IPv4) and C(IPv6).
            returned: always
            type: str
            sample: IPv4
        ip_tags:
            description:
                - The list of tags associated with the public IP prefixes.
            returned: always
            type: list
            sample: [{'type': 'FirstPartyUsage', 'value': 'Storage'}]
        resource_guid:
            description:
                - The resource GUID property of the public IP prefix resource.
            type: str
            sample: "47cafa04-851d-4579-894d-74ad6afe3233"
        custom_ip_prefix:
            description:
                - The customIpPrefix that this prefix is associated with.
            type: dict
            returned: always
            sample: {}
        public_ip_addresses:
            description:
                - The list of all referenced PublicIPAddresses.
            type: list
            sample: []
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


def prefix_to_dict(prefix):
    result = dict(
        id=prefix.id,
        name=prefix.name,
        tags=prefix.tags,
        type=prefix.type,
        location=prefix.location,
        public_ip_address_version=prefix.public_ip_address_version,
        prefix_length=prefix.prefix_length,
        provisioning_state=prefix.provisioning_state,
        etag=prefix.etag,
        zones=prefix.zones,
        sku=dict(),
        ip_tags=list(),
        custom_ip_prefix=dict(),
        ip_prefix=prefix.ip_prefix,
        resource_guid=prefix.resource_guid,
        public_ip_addresses=list(),
        extended_location=prefix.extended_location
    )
    if prefix.public_ip_addresses:
        result['public_ip_addresses'] = [x.id for x in prefix.public_ip_addresses]
    if prefix.sku:
        result['sku']['name'] = prefix.sku.name
        result['sku']['tier'] = prefix.sku.tier
    if prefix.custom_ip_prefix:
        result['custom_ip_prefix']['id'] = prefix.custom_ip_prefix.id
    if prefix.ip_tags:
        result['ip_tags'] = [dict(type=x.ip_tag_type, value=x.tag) for x in prefix.ip_tags]
    return result


class AzureRMPublicIPPrefix(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            name=dict(type='str', required=True),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            location=dict(type='str'),
            public_ip_address_version=dict(type='str', choices=['IPV4', 'IPV6']),
            extended_location=dict(type='str'),
            prefix_length=dict(type='int'),
            custom_ip_prefix=dict(
                type='dict',
                options=dict(
                    id=dict(type='str')
                )
            ),
            sku=dict(
                type='dict',
                options=dict(
                    name=dict(type='str', choices=['Standard']),
                    tier=dict(type='str', choices=['Regional', 'Global'])
                )
            ),
            ip_tags=dict(
                type='list',
                elements='dict',
                options=dict(
                    ip_tag_type=dict(type='str'),
                    tag=dict(type='str')
                )
            ),
            zones=dict(type='list', elements='str', choices=['1', '2', '3'])
        )

        self.resource_group = None
        self.name = None
        self.location = None
        self.state = None
        self.tags = None
        self.zones = None
        self.sku = None
        self.ip_tags = None
        self.public_ip_address_version = None
        self.prefix_length = None
        self.custom_ip_prefix = None

        self.results = dict(
            changed=False,
            state=dict()
        )

        super(AzureRMPublicIPPrefix, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                    supports_check_mode=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        results = dict()
        changed = False
        prefix = None
        update_tags = False

        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            # Set default location
            self.location = resource_group.location

        try:
            self.log("Fetch public ip prefix {0}".format(self.name))
            prefix = self.network_client.public_ip_prefixes.get(self.resource_group, self.name)
            self.log("Public IP prefix {0} exists".format(self.name))

            if self.state == 'present':
                results = prefix_to_dict(prefix)

                if self.public_ip_address_version is not None and \
                   self.public_ip_address_version.lower() != results['public_ip_address_version'].lower():
                    changed = False
                    results['public_ip_address_version'] = self.public_ip_address_version
                    self.fail("The public_ip_address_version can't be updated")

                if self.prefix_length is not None and self.prefix_length != results['prefix_length']:
                    changed = False
                    results['prefix_length'] = self.prefix_length
                    self.fail("The prefix_length can't be updated")

                if self.sku is not None:
                    for key in self.sku.keys():
                        if self.sku[key] != results['sku'].get(key):
                            changed = False
                            self.fail("The sku can't be updated")
                            results['sku'] = self.sku

                if self.zones is not None and not all(key in results['zones'] for key in self.zones):
                    changed = False
                    results['zones'] = self.zones
                    self.fail("The zone can't be updated")

                if self.extended_location is not None and self.extended_location != results['extended_location']:
                    changed = False
                    results['extended_location'] = self.extended_location
                    self.fail("The extended_location can't be updated")

                if self.ip_tags is not None:
                    for key in self.ip_tags.keys():
                        if self.ip_tags[key] != results['ip_tags'].get(key):
                            changed = False
                            results['ip_tags'] = self.ip_tags
                            self.fail("The ip_tags can't be updated")

                if self.custom_ip_prefix is not None:
                    if results.get('custom_ip_prefix') is None:
                        changed = False
                        results['custom_ip_prefix'] = self.custom_ip_prefix
                        self.fail("The custom_ip_prefix can't be updated")
                    elif self.custom_ip_prefix['id'].lower() != results['custom_ip_prefix']['id'].lower():
                        changed = False
                        results['custom_ip_prefix'] = self.custom_ip_prefix
                        self.fail("The custom_ip_prefix can't be updated")

                update_tags, results['tags'] = self.update_tags(results['tags'])
                if update_tags:
                    self.log("CHANGED: tags")
                    changed = True
                self.tags = results['tags']

            elif self.state == 'absent':
                self.log("CHANGED: public ip prefix {0} exists but requested state is 'absent'".format(self.name))
                changed = True
        except ResourceNotFoundError:
            self.log('Public ip prefix {0} does not exist'.format(self.name))
            if self.state == 'present':
                self.log("CHANGED: public IP prefix {0} does not exist but requested state is 'present'".format(self.name))
                changed = True

        self.results['state'] = results
        self.results['changed'] = changed

        if self.check_mode:
            results['changed'] = True
            return results

        if update_tags:
            self.results['state'] = self.update_prefix_tags(self.tags)

        elif changed:
            if self.state == 'present':
                self.log("Create or Update Public IP prefix {0}".format(self.name))
                prefix = self.network_models.PublicIPPrefix(
                    location=self.location,
                    public_ip_address_version=self.public_ip_address_version,
                    prefix_length=self.prefix_length,
                    zones=self.zones,
                    tags=self.tags,
                    sku=self.sku,
                    ip_tags=self.ip_tags,
                    custom_ip_prefix=self.custom_ip_prefix
                )
                self.results['state'] = self.create_or_update_prefix(prefix)

            elif self.state == 'absent':
                self.log('Delete public ip {0}'.format(self.name))
                self.delete_prefix()

        return self.results

    def update_prefix_tags(self, tags):
        try:
            prefix = self.network_client.public_ip_prefixes.update_tags(self.resource_group, self.name, dict(tags=tags))
        except Exception as exc:
            self.fail("Error updating tags {0} - {1}".format(self.name, str(exc)))
        return prefix_to_dict(prefix)

    def create_or_update_prefix(self, prefix):
        try:
            poller = self.network_client.public_ip_prefixes.begin_create_or_update(self.resource_group, self.name, prefix)
            prefix = self.get_poller_result(poller)
        except Exception as exc:
            self.fail("Error creating or updating {0} - {1}".format(self.name, str(exc)))
        return prefix_to_dict(prefix)

    def delete_prefix(self):
        try:
            poller = self.network_client.public_ip_prefixes.begin_delete(self.resource_group, self.name)
            self.get_poller_result(poller)
        except Exception as exc:
            self.fail("Error deleting {0} - {1}".format(self.name, str(exc)))

        self.results['state']['status'] = 'Deleted'
        return True


def main():
    AzureRMPublicIPPrefix()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: client_proximity_v1_info
short_description: Information module for Client Proximity V1
description:
- Get all Client Proximity V1.
- >
   This intent API will provide client proximity information for a specific wireless user. Proximity is defined as
   presence on the same floor at the same time as the specified wireless user. The Proximity workflow requires the
   subscription to the following event via the Event Notification workflow prior to making this API call NETWORK-
   CLIENTS-3-506 - Client Proximity Report.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  username:
    description:
    - Username query parameter. Wireless client username for which proximity information is required.
    type: str
  number_days:
    description:
    - >
      Number_days query parameter. Number of days to track proximity until current date. Defaults and maximum up
      to 14 days.
    type: float
  time_resolution:
    description:
    - >
      Time_resolution query parameter. Time interval (in minutes) to measure proximity. Defaults to 15 minutes
      with a minimum 5 minutes.
    type: float
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Clients ClientProximityV1
  description: Complete reference of the ClientProximityV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!client-proximity
notes:
  - SDK Method used are
    clients.Clients.client_proximity_v1,

  - Paths used are
    get /dna/intent/api/v1/client-proximity,

"""

EXAMPLES = r"""
- name: Get all Client Proximity V1
  cisco.dnac.client_proximity_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    username: string
    number_days: 0
    time_resolution: 0
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "executionId": "string",
      "executionStatusUrl": "string",
      "message": "string"
    }
"""



# -*- coding: utf-8 -*-

# (c) 2023 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
name: aws_collection_constants
author:
  - Mark Chappell (@tremble)
short_description: expose various collection related constants
version_added: 6.0.0
description:
  - Exposes various collection related constants for use in integration tests.
options:
  _terms:
    description: Name of the constant.
    choices:
      - MINIMUM_BOTOCORE_VERSION
      - MINIMUM_BOTO3_VERSION
      - HAS_BOTO3
      - AMAZON_AWS_COLLECTION_VERSION
      - AMAZON_AWS_COLLECTION_NAME
      - COMMUNITY_AWS_COLLECTION_VERSION
      - COMMUNITY_AWS_COLLECTION_NAME
    required: True
"""

EXAMPLES = r"""
"""

RETURN = r"""
_raw:
  description: value
  type: str
"""

from ansible.errors import AnsibleLookupError
from ansible.plugins.lookup import LookupBase

import ansible_collections.amazon.aws.plugins.module_utils.botocore as botocore_utils
import ansible_collections.amazon.aws.plugins.module_utils.common as common_utils

try:
    import ansible_collections.community.aws.plugins.module_utils.common as community_utils

    HAS_COMMUNITY = True
except ImportError:
    HAS_COMMUNITY = False


class LookupModule(LookupBase):
    def lookup_constant(self, name):  # pylint: disable=too-many-return-statements
        if name == "MINIMUM_BOTOCORE_VERSION":
            return botocore_utils.MINIMUM_BOTOCORE_VERSION
        if name == "MINIMUM_BOTO3_VERSION":
            return botocore_utils.MINIMUM_BOTO3_VERSION
        if name == "HAS_BOTO3":
            return botocore_utils.HAS_BOTO3

        if name == "AMAZON_AWS_COLLECTION_VERSION":
            return common_utils.AMAZON_AWS_COLLECTION_VERSION
        if name == "AMAZON_AWS_COLLECTION_NAME":
            return common_utils.AMAZON_AWS_COLLECTION_NAME

        if name == "COMMUNITY_AWS_COLLECTION_VERSION":
            if not HAS_COMMUNITY:
                raise AnsibleLookupError("Unable to load ansible_collections.community.aws.plugins.module_utils.common")
            return community_utils.COMMUNITY_AWS_COLLECTION_VERSION
        if name == "COMMUNITY_AWS_COLLECTION_NAME":
            if not HAS_COMMUNITY:
                raise AnsibleLookupError("Unable to load ansible_collections.community.aws.plugins.module_utils.common")
            return community_utils.COMMUNITY_AWS_COLLECTION_NAME

    def run(self, terms, variables, **kwargs):
        self.set_options(var_options=variables, direct=kwargs)
        if not terms:
            raise AnsibleLookupError("Constant name not provided")
        if len(terms) > 1:
            raise AnsibleLookupError("Multiple constant names provided")
        name = terms[0].upper()

        return [self.lookup_constant(name)]



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_extranet_policies_v1
short_description: Resource module for Sda Extranet Policies V1
description:
- Manage operations create, update and delete of the resource Sda Extranet Policies V1.
- Adds an extranet policy based on user input.
- Deletes an extranet policy based on id.
- Deletes extranet policies based on user input.
- Updates an extranet policy based on user input.
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  extranetPolicyName:
    description: ExtranetPolicyName query parameter. Name of the extranet policy.
    type: str
  id:
    description: Id path parameter. ID of the extranet policy.
    type: str
  payload:
    description: Sda Extranet Policies's payload.
    elements: dict
    suboptions:
      extranetPolicyName:
        description: Name of the existing extranet policy (updating this field is not
          allowed).
        type: str
      fabricIds:
        description: IDs of the fabric sites associated with this extranet policy.
        elements: str
        type: list
      id:
        description: ID of the existing extranet policy (updating this field is not
          allowed).
        type: str
      providerVirtualNetworkName:
        description: Name of the existing provider virtual network (updating this field
          is not allowed).
        type: str
      subscriberVirtualNetworkNames:
        description: Name of the subscriber virtual networks.
        elements: str
        type: list
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA AddExtranetPolicyV1
  description: Complete reference of the AddExtranetPolicyV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-extranet-policy
- name: Cisco DNA Center documentation for SDA DeleteExtranetPoliciesV1
  description: Complete reference of the DeleteExtranetPoliciesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-extranet-policies
- name: Cisco DNA Center documentation for SDA DeleteExtranetPolicyByIdV1
  description: Complete reference of the DeleteExtranetPolicyByIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-extranet-policy-by-id
- name: Cisco DNA Center documentation for SDA UpdateExtranetPolicyV1
  description: Complete reference of the UpdateExtranetPolicyV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-extranet-policy
notes:
  - SDK Method used are
    sda.Sda.add_extranet_policy_v1,
    sda.Sda.delete_extranet_policy_by_id_v1,
    sda.Sda.update_extranet_policy_v1,

  - Paths used are
    post /dna/intent/api/v1/sda/extranetPolicies,
    delete /dna/intent/api/v1/sda/extranetPolicies,
    delete /dna/intent/api/v1/sda/extranetPolicies/{id},
    put /dna/intent/api/v1/sda/extranetPolicies,

"""

EXAMPLES = r"""
- name: Delete all
  cisco.dnac.sda_extranet_policies_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    extranetPolicyName: string

- name: Update all
  cisco.dnac.sda_extranet_policies_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - extranetPolicyName: string
      fabricIds:
      - string
      id: string
      providerVirtualNetworkName: string
      subscriberVirtualNetworkNames:
      - string

- name: Create
  cisco.dnac.sda_extranet_policies_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - extranetPolicyName: string
      fabricIds:
      - string
      providerVirtualNetworkName: string
      subscriberVirtualNetworkNames:
      - string

- name: Delete by id
  cisco.dnac.sda_extranet_policies_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#
# (c) 2019, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type
import json

from threading import RLock

from ansible.module_utils.six import itervalues
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.config import (
    NetworkConfig,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import to_list


_registered_providers = {}
_provider_lock = RLock()


def register_provider(network_os, module_name):
    def wrapper(cls):
        _provider_lock.acquire()
        try:
            if network_os not in _registered_providers:
                _registered_providers[network_os] = {}
            for ct in cls.supported_connections:
                if ct not in _registered_providers[network_os]:
                    _registered_providers[network_os][ct] = {}
            for item in to_list(module_name):
                for entry in itervalues(_registered_providers[network_os]):
                    entry[item] = cls
        finally:
            _provider_lock.release()
        return cls

    return wrapper


def get(network_os, module_name, connection_type):
    network_os_providers = _registered_providers.get(network_os)
    if network_os_providers is None:
        raise ValueError("unable to find a suitable provider for this module")
    if connection_type not in network_os_providers:
        raise ValueError("provider does not support this connection type")
    elif module_name not in network_os_providers[connection_type]:
        raise ValueError("could not find a suitable provider for this module")
    return network_os_providers[connection_type][module_name]


class ProviderBase(object):
    supported_connections = ()

    def __init__(self, params, connection=None, check_mode=False):
        self.params = params
        self.connection = connection
        self.check_mode = check_mode

    @property
    def capabilities(self):
        if not hasattr(self, "_capabilities"):
            resp = self.from_json(self.connection.get_capabilities())
            setattr(self, "_capabilities", resp)
        return getattr(self, "_capabilities")

    def get_value(self, path):
        params = self.params.copy()
        for key in path.split("."):
            params = params[key]
        return params

    def get_facts(self, subset=None):
        raise NotImplementedError(self.__class__.__name__)

    def edit_config(self):
        raise NotImplementedError(self.__class__.__name__)


class CliProvider(ProviderBase):
    supported_connections = ("network_cli",)

    @property
    def capabilities(self):
        if not hasattr(self, "_capabilities"):
            resp = self.from_json(self.connection.get_capabilities())
            setattr(self, "_capabilities", resp)
        return getattr(self, "_capabilities")

    def get_config_context(self, config, path, indent=2):
        if config is not None:
            netcfg = NetworkConfig(indent=indent, contents=config)
            try:
                config = netcfg.get_block_config(to_list(path))
            except ValueError:
                config = None
            return config

    def render(self, config=None):
        raise NotImplementedError(self.__class__.__name__)

    def cli(self, command):
        try:
            if not hasattr(self, "_command_output"):
                setattr(self, "_command_output", {})
            return self._command_output[command]
        except KeyError:
            out = self.connection.get(command)
            try:
                out = json.loads(out)
            except ValueError:
                pass
            self._command_output[command] = out
            return out

    def get_facts(self, subset=None):
        return self.populate()

    def edit_config(self, config=None):
        commands = self.render(config)
        if commands and self.check_mode is False:
            self.connection.edit_config(commands)
        return commands



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
from ansible.plugins.action import ActionBase
try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator,
    )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
from ansible_collections.cisco.dnac.plugins.plugin_utils.dnac import (
    DNACSDK,
    dnac_argument_spec,
    dnac_compare_equality,
    get_dict_result,
)
from ansible_collections.cisco.dnac.plugins.plugin_utils.exceptions import (
    InconsistentParameters,
)

# Get common arguments specification
argument_spec = dnac_argument_spec()
# Add arguments specific for this module
argument_spec.update(dict(
    state=dict(type="str", default="present", choices=["present", "absent"]),
    name=dict(type="str"),
    securityLevel=dict(type="str"),
    passphrase=dict(type="str"),
    enableFastLane=dict(type="bool"),
    enableMACFiltering=dict(type="bool"),
    trafficType=dict(type="str"),
    radioPolicy=dict(type="str"),
    enableBroadcastSSID=dict(type="bool"),
    fastTransition=dict(type="str"),
    enableSessionTimeOut=dict(type="bool"),
    sessionTimeOut=dict(type="int"),
    enableClientExclusion=dict(type="bool"),
    clientExclusionTimeout=dict(type="int"),
    enableBasicServiceSetMaxIdle=dict(type="bool"),
    basicServiceSetClientIdleTimeout=dict(type="int"),
    enableDirectedMulticastService=dict(type="bool"),
    enableNeighborList=dict(type="bool"),
    mfpClientProtection=dict(type="str"),
    nasOptions=dict(type="list"),
    profileName=dict(type="str"),
    policyProfileName=dict(type="str"),
    aaaOverride=dict(type="bool"),
    coverageHoleDetectionEnable=dict(type="bool"),
    protectedManagementFrame=dict(type="str"),
    multiPSKSettings=dict(type="list"),
    clientRateLimit=dict(type="float"),
    authKeyMgmt=dict(type="list"),
    rsnCipherSuiteGcmp256=dict(type="bool"),
    rsnCipherSuiteCcmp256=dict(type="bool"),
    rsnCipherSuiteGcmp128=dict(type="bool"),
    ghz6PolicyClientSteering=dict(type="bool"),
    ghz24Policy=dict(type="str"),
    ssidName=dict(type="str"),
))

required_if = [
    ("state", "present", ["name", "ssidName"], True),
    ("state", "absent", ["name", "ssidName"], True),
]
required_one_of = []
mutually_exclusive = []
required_together = []


class WirelessEnterpriseSsidV1(object):
    def __init__(self, params, dnac):
        self.dnac = dnac
        self.new_object = dict(
            name=params.get("name"),
            securityLevel=params.get("securityLevel"),
            passphrase=params.get("passphrase"),
            enableFastLane=params.get("enableFastLane"),
            enableMACFiltering=params.get("enableMACFiltering"),
            trafficType=params.get("trafficType"),
            radioPolicy=params.get("radioPolicy"),
            enableBroadcastSSID=params.get("enableBroadcastSSID"),
            fastTransition=params.get("fastTransition"),
            enableSessionTimeOut=params.get("enableSessionTimeOut"),
            sessionTimeOut=params.get("sessionTimeOut"),
            enableClientExclusion=params.get("enableClientExclusion"),
            clientExclusionTimeout=params.get("clientExclusionTimeout"),
            enableBasicServiceSetMaxIdle=params.get("enableBasicServiceSetMaxIdle"),
            basicServiceSetClientIdleTimeout=params.get("basicServiceSetClientIdleTimeout"),
            enableDirectedMulticastService=params.get("enableDirectedMulticastService"),
            enableNeighborList=params.get("enableNeighborList"),
            mfpClientProtection=params.get("mfpClientProtection"),
            nasOptions=params.get("nasOptions"),
            profileName=params.get("profileName"),
            policyProfileName=params.get("policyProfileName"),
            aaaOverride=params.get("aaaOverride"),
            coverageHoleDetectionEnable=params.get("coverageHoleDetectionEnable"),
            protectedManagementFrame=params.get("protectedManagementFrame"),
            multiPSKSettings=params.get("multiPSKSettings"),
            clientRateLimit=params.get("clientRateLimit"),
            authKeyMgmt=params.get("authKeyMgmt"),
            rsnCipherSuiteGcmp256=params.get("rsnCipherSuiteGcmp256"),
            rsnCipherSuiteCcmp256=params.get("rsnCipherSuiteCcmp256"),
            rsnCipherSuiteGcmp128=params.get("rsnCipherSuiteGcmp128"),
            ghz6PolicyClientSteering=params.get("ghz6PolicyClientSteering"),
            ghz24Policy=params.get("ghz24Policy"),
            ssid_name=params.get("ssidName"),
        )

    def get_all_params(self, name=None, id=None):
        new_object_params = {}
        new_object_params['ssid_name'] = self.new_object.get('ssidName') or \
            self.new_object.get('ssid_name')
        return new_object_params

    def create_params(self):
        new_object_params = {}
        new_object_params['name'] = self.new_object.get('name')
        new_object_params['securityLevel'] = self.new_object.get('securityLevel')
        new_object_params['passphrase'] = self.new_object.get('passphrase')
        new_object_params['enableFastLane'] = self.new_object.get('enableFastLane')
        new_object_params['enableMACFiltering'] = self.new_object.get('enableMACFiltering')
        new_object_params['trafficType'] = self.new_object.get('trafficType')
        new_object_params['radioPolicy'] = self.new_object.get('radioPolicy')
        new_object_params['enableBroadcastSSID'] = self.new_object.get('enableBroadcastSSID')
        new_object_params['fastTransition'] = self.new_object.get('fastTransition')
        new_object_params['enableSessionTimeOut'] = self.new_object.get('enableSessionTimeOut')
        new_object_params['sessionTimeOut'] = self.new_object.get('sessionTimeOut')
        new_object_params['enableClientExclusion'] = self.new_object.get('enableClientExclusion')
        new_object_params['clientExclusionTimeout'] = self.new_object.get('clientExclusionTimeout')
        new_object_params['enableBasicServiceSetMaxIdle'] = self.new_object.get('enableBasicServiceSetMaxIdle')
        new_object_params['basicServiceSetClientIdleTimeout'] = self.new_object.get('basicServiceSetClientIdleTimeout')
        new_object_params['enableDirectedMulticastService'] = self.new_object.get('enableDirectedMulticastService')
        new_object_params['enableNeighborList'] = self.new_object.get('enableNeighborList')
        new_object_params['mfpClientProtection'] = self.new_object.get('mfpClientProtection')
        new_object_params['nasOptions'] = self.new_object.get('nasOptions')
        new_object_params['profileName'] = self.new_object.get('profileName')
        new_object_params['policyProfileName'] = self.new_object.get('policyProfileName')
        new_object_params['aaaOverride'] = self.new_object.get('aaaOverride')
        new_object_params['coverageHoleDetectionEnable'] = self.new_object.get('coverageHoleDetectionEnable')
        new_object_params['protectedManagementFrame'] = self.new_object.get('protectedManagementFrame')
        new_object_params['multiPSKSettings'] = self.new_object.get('multiPSKSettings')
        new_object_params['clientRateLimit'] = self.new_object.get('clientRateLimit')
        new_object_params['authKeyMgmt'] = self.new_object.get('authKeyMgmt')
        new_object_params['rsnCipherSuiteGcmp256'] = self.new_object.get('rsnCipherSuiteGcmp256')
        new_object_params['rsnCipherSuiteCcmp256'] = self.new_object.get('rsnCipherSuiteCcmp256')
        new_object_params['rsnCipherSuiteGcmp128'] = self.new_object.get('rsnCipherSuiteGcmp128')
        new_object_params['ghz6PolicyClientSteering'] = self.new_object.get('ghz6PolicyClientSteering')
        new_object_params['ghz24Policy'] = self.new_object.get('ghz24Policy')
        return new_object_params

    def delete_by_name_params(self):
        new_object_params = {}
        new_object_params['ssid_name'] = self.new_object.get('ssid_name')
        return new_object_params

    def update_all_params(self):
        new_object_params = {}
        new_object_params['name'] = self.new_object.get('name')
        new_object_params['securityLevel'] = self.new_object.get('securityLevel')
        new_object_params['passphrase'] = self.new_object.get('passphrase')
        new_object_params['enableFastLane'] = self.new_object.get('enableFastLane')
        new_object_params['enableMACFiltering'] = self.new_object.get('enableMACFiltering')
        new_object_params['trafficType'] = self.new_object.get('trafficType')
        new_object_params['radioPolicy'] = self.new_object.get('radioPolicy')
        new_object_params['enableBroadcastSSID'] = self.new_object.get('enableBroadcastSSID')
        new_object_params['fastTransition'] = self.new_object.get('fastTransition')
        new_object_params['enableSessionTimeOut'] = self.new_object.get('enableSessionTimeOut')
        new_object_params['sessionTimeOut'] = self.new_object.get('sessionTimeOut')
        new_object_params['enableClientExclusion'] = self.new_object.get('enableClientExclusion')
        new_object_params['clientExclusionTimeout'] = self.new_object.get('clientExclusionTimeout')
        new_object_params['enableBasicServiceSetMaxIdle'] = self.new_object.get('enableBasicServiceSetMaxIdle')
        new_object_params['basicServiceSetClientIdleTimeout'] = self.new_object.get('basicServiceSetClientIdleTimeout')
        new_object_params['enableDirectedMulticastService'] = self.new_object.get('enableDirectedMulticastService')
        new_object_params['enableNeighborList'] = self.new_object.get('enableNeighborList')
        new_object_params['mfpClientProtection'] = self.new_object.get('mfpClientProtection')
        new_object_params['nasOptions'] = self.new_object.get('nasOptions')
        new_object_params['profileName'] = self.new_object.get('profileName')
        new_object_params['policyProfileName'] = self.new_object.get('policyProfileName')
        new_object_params['aaaOverride'] = self.new_object.get('aaaOverride')
        new_object_params['coverageHoleDetectionEnable'] = self.new_object.get('coverageHoleDetectionEnable')
        new_object_params['protectedManagementFrame'] = self.new_object.get('protectedManagementFrame')
        new_object_params['multiPSKSettings'] = self.new_object.get('multiPSKSettings')
        new_object_params['clientRateLimit'] = self.new_object.get('clientRateLimit')
        new_object_params['authKeyMgmt'] = self.new_object.get('authKeyMgmt')
        new_object_params['rsnCipherSuiteGcmp256'] = self.new_object.get('rsnCipherSuiteGcmp256')
        new_object_params['rsnCipherSuiteCcmp256'] = self.new_object.get('rsnCipherSuiteCcmp256')
        new_object_params['rsnCipherSuiteGcmp128'] = self.new_object.get('rsnCipherSuiteGcmp128')
        new_object_params['ghz6PolicyClientSteering'] = self.new_object.get('ghz6PolicyClientSteering')
        new_object_params['ghz24Policy'] = self.new_object.get('ghz24Policy')
        return new_object_params

    def get_object_by_name(self, name):
        result = None
        # NOTE: Does not have a get by name method or it is in another action
        try:
            items = self.dnac.exec(
                family="wireless",
                function="get_enterprise_ssid_v1",
                params=self.get_all_params(name=name),
            )
            if isinstance(items, dict):
                if 'response' in items:
                    items = items.get('response')
            result = get_dict_result(items, 'name', name)
        except Exception:
            result = None
        return result

    def get_object_by_id(self, id):
        result = None
        # NOTE: Does not have a get by id method or it is in another action
        try:
            items = self.dnac.exec(
                family="wireless",
                function="get_enterprise_ssid_v1",
                params=self.get_all_params(id=id),
            )
            if isinstance(items, dict):
                if 'response' in items:
                    items = items.get('response')
            result = get_dict_result(items, 'id', id)
        except Exception:
            result = None
        return result

    def exists(self):
        id_exists = False
        name_exists = False
        prev_obj = None
        o_id = self.new_object.get("id")
        name = self.new_object.get("name")
        name = name or self.new_object.get("ssid_name")
        if o_id:
            prev_obj = self.get_object_by_id(o_id)
            id_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if not id_exists and name:
            prev_obj = self.get_object_by_name(name)
            name_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if id_exists:
            _name = prev_obj.get("name")
            _name = _name or prev_obj.get("ssidName")
            if _name:
                self.new_object.update(dict(ssid_name=_name))
        if name_exists:
            _id = prev_obj.get("id")
            if id_exists and name_exists and o_id != _id:
                raise InconsistentParameters("The 'id' and 'name' params don't refer to the same object")
            if _id:
                self.new_object.update(dict(id=_id))
        it_exists = prev_obj is not None and isinstance(prev_obj, dict)
        return (it_exists, prev_obj)

    def requires_update(self, current_obj):
        requested_obj = self.new_object

        obj_params = [
            ("name", "name"),
            ("securityLevel", "securityLevel"),
            ("passphrase", "passphrase"),
            ("enableFastLane", "enableFastLane"),
            ("enableMACFiltering", "enableMACFiltering"),
            ("trafficType", "trafficType"),
            ("radioPolicy", "radioPolicy"),
            ("enableBroadcastSSID", "enableBroadcastSSID"),
            ("fastTransition", "fastTransition"),
            ("enableSessionTimeOut", "enableSessionTimeOut"),
            ("sessionTimeOut", "sessionTimeOut"),
            ("enableClientExclusion", "enableClientExclusion"),
            ("clientExclusionTimeout", "clientExclusionTimeout"),
            ("enableBasicServiceSetMaxIdle", "enableBasicServiceSetMaxIdle"),
            ("basicServiceSetClientIdleTimeout", "basicServiceSetClientIdleTimeout"),
            ("enableDirectedMulticastService", "enableDirectedMulticastService"),
            ("enableNeighborList", "enableNeighborList"),
            ("mfpClientProtection", "mfpClientProtection"),
            ("nasOptions", "nasOptions"),
            ("profileName", "profileName"),
            ("policyProfileName", "policyProfileName"),
            ("aaaOverride", "aaaOverride"),
            ("coverageHoleDetectionEnable", "coverageHoleDetectionEnable"),
            ("protectedManagementFrame", "protectedManagementFrame"),
            ("multiPSKSettings", "multiPSKSettings"),
            ("clientRateLimit", "clientRateLimit"),
            ("authKeyMgmt", "authKeyMgmt"),
            ("rsnCipherSuiteGcmp256", "rsnCipherSuiteGcmp256"),
            ("rsnCipherSuiteCcmp256", "rsnCipherSuiteCcmp256"),
            ("rsnCipherSuiteGcmp128", "rsnCipherSuiteGcmp128"),
            ("ghz6PolicyClientSteering", "ghz6PolicyClientSteering"),
            ("ghz24Policy", "ghz24Policy"),
            ("ssidName", "ssid_name"),
        ]
        # Method 1. Params present in request (Ansible) obj are the same as the current (DNAC) params
        # If any does not have eq params, it requires update
        return any(not dnac_compare_equality(current_obj.get(dnac_param),
                                             requested_obj.get(ansible_param))
                   for (dnac_param, ansible_param) in obj_params)

    def create(self):
        result = self.dnac.exec(
            family="wireless",
            function="create_enterprise_ssid_v1",
            params=self.create_params(),
            op_modifies=True,
        )
        return result

    def update(self):
        id = self.new_object.get("id")
        name = self.new_object.get("name")
        result = None
        result = self.dnac.exec(
            family="wireless",
            function="update_enterprise_ssid_v1",
            params=self.update_all_params(),
            op_modifies=True,
        )
        return result

    def delete(self):
        id = self.new_object.get("id")
        name = self.new_object.get("name")
        name = name or self.new_object.get("ssid_name")
        result = None
        if not name:
            prev_obj_id = self.get_object_by_id(id)
            name_ = None
            if prev_obj_id:
                name_ = prev_obj_id.get("name")
                name_ = name_ or prev_obj_id.get("ssidName")
            if name_:
                self.new_object.update(dict(ssid_name=name_))
        result = self.dnac.exec(
            family="wireless",
            function="delete_enterprise_ssid_v1",
            params=self.delete_by_name_params(),
        )
        return result


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail("ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'")
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = False
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()

        dnac = DNACSDK(self._task.args)
        obj = WirelessEnterpriseSsidV1(self._task.args, dnac)

        state = self._task.args.get("state")

        response = None

        if state == "present":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                if obj.requires_update(prev_obj):
                    response = obj.update()
                    dnac.object_updated()
                else:
                    response = prev_obj
                    dnac.object_already_present()
            else:
                response = obj.create()
                dnac.object_created()

        elif state == "absent":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                response = obj.delete()
                dnac.object_deleted()
            else:
                dnac.object_already_absent()

        self._result.update(dict(dnac_response=response))
        self._result.update(dnac.exit_json())
        return self._result



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for iosxr_prefix_lists
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: iosxr_prefix_lists
short_description: Resource module to configure prefix lists.
description:
- This module manages prefix-lists configuration on devices running Cisco IOSXR.
version_added: 2.3.0
notes:
- Tested against IOSXR 7.0.2.
- This module works with connection C(network_cli).
author: Ashwini Mhatre (@amhatre)
options:
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the Iosxr device by
        executing the command B(show running-config prefix-list).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  config:
    description: A list of prefix-lists configuration.
    type: list
    elements: dict
    suboptions:
      afi:
        description:
        - The Address Family Identifier (AFI) for the prefix-lists.
        type: str
        choices: ["ipv4", "ipv6"]
      prefix_lists:
        description: List of prefix-list configurations.
        type: list
        elements: dict
        suboptions:
          name:
            description: Name of the prefix-list.
            type: str
          entries:
            description: List of configurations for the specified prefix-list
            type: list
            elements: dict
            suboptions:
              sequence:
                description: Sequence Number.
                type: int
              action:
                description: Prefix-List permit or deny.
                type: str
                choices: ["permit", "deny", "remark"]
              description:
                description: Description of the prefix list. only applicable for action "remark".
                type: str
              prefix:
                description: IP or IPv6 prefix in A.B.C.D/LEN or A:B::C:D/LEN format. only applicable for action "permit" and "deny"
                type: str
              eq:
                description: Exact prefix length to be matched.
                type: int
              ge:
                description: Minimum prefix length to be matched.
                type: int
              le:
                description: Maximum prefix length to be matched.
                type: int
  state:
    description:
    - The state the configuration should be left in.
    - Refer to examples for more details.
    - With state I(replaced), for the listed prefix-lists,
      sequences that are in running-config but not in the task are negated.
    - With state I(overridden), all prefix-lists that are in running-config but
      not in the task are negated.
    - Please refer to examples for more details.
    type: str
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - parsed
    - gathered
    - rendered
    default: merged
"""

EXAMPLES = """
# Using merged


# Before state
# RP/0/0/CPU0:10#show running-config
# Thu Feb  4 09:38:36.245 UTC
# % No such configuration item(s)
# RP/0/0/CPU0:10#
#


- name: Merge the provided configuration with the existing running configuration
  cisco.iosxr.iosxr_prefix_lists:
    state: merged
    config:
      - afi: ipv6
        prefix_lists:
          - name: pl_1
            entries:
              - prefix: '2001:db8:1234::/48'
                action: deny
                sequence: 1
          - name: pl_2
            entries:
              - sequence: 2
                action: remark
                description: TEST_PL_2_REMARK
      - afi: ipv4
        prefix_lists:
          - name: pl1
            entries:
              - sequence: 3
                action: remark
                description: TEST_PL1_2_REMARK
              - sequence: 4
                action: permit
                prefix: 10.0.0.0/24
          - name: pl2
            entries:
              - sequence: 5
                action: remark
                description: TEST_PL2_REMARK
          - name: pl3
            entries:
              - sequence: 6
                action: permit
                prefix: 35.0.0.0/8
                eq: 0

# Task Output
# -------------
# before: []
# commands:
# - ipv6 prefix-list pl_1 1 deny 2001:db8:1234::/48
# - ipv6 prefix-list pl_2 2 remark TEST_PL_2_REMARK
# - ipv4 prefix-list pl1 3 remark TEST_PL1_2_REMARK
# - ipv4 prefix-list pl1 4 permit 10.0.0.0/24
# - ipv4 prefix-list pl2 5 remark TEST_PL2_REMARK
# - ipv4 prefix-list pl3 6 permit 35.0.0.0/8 eq 0
# after:
# - afi: ipv6
#   prefix_lists:
#   - name: pl_1
#     entries:
#     - prefix: 2001:db8:1234::/48
#       action: deny
#       sequence: 1
#   - name: pl_2
#     entries:
#     - sequence: 2
#       action: remark
#       description: TEST_PL_2_REMARK
# - afi: ipv4
#   prefix_lists:
#   - name: pl1
#     entries:
#     - sequence: 3
#       action: remark
#       description: TEST_PL1_2_REMARK
#     - sequence: 4
#       action: permit
#       prefix: 10.0.0.0/24
#   - name: pl2
#     entries:
#     - sequence: 5
#       action: remark
#       description: TEST_PL2_REMARK
#   - name: pl3
#     entries:
#     - sequence: 6
#       action: permit
#       prefix: 35.0.0.0/8
#       eq: 0


# After state:
# ------------
# RP/0/0/CPU0:10#show running-config
# ipv6 prefix-list pl_1
#  1 deny 2001:db8:1234::/48
# !
# ipv6 prefix-list pl_2
#  2 remark TEST_PL_2_REMAR
# !
# ipv4 prefix-list pl1
#  3 remark TEST_PL1_2_REMARK
#  4 permit 10.0.0.0/24
# !
# ipv4 prefix-list pl2
#  5 remark TEST_PL2_REMARK
# !
# ipv4 prefix-list pl3
#  6 permit 35.0.0.0/8 eq 0
# !


# Using replaced:


# Before state:
# -------------
# RP/0/0/CPU0:10#show running-config
#
# ipv6 prefix-list pl_1
#  1 deny 2001:db8:1234::/48
# !
# ipv6 prefix-list pl_2
#  2 remark TEST_PL_2_REMARK
# !
# ipv4 prefix-list pl1
#  3 remark TEST_PL1_2_REMARK
#  4 permit 10.0.0.0/24
# !
# ipv4 prefix-list pl2
#  5 remark TEST_PL2_REMARK
# !
#


- name: >-
    Replace device configurations of listed prefix lists with provided
    configurations
  register: result
  cisco.iosxr.iosxr_prefix_lists:
    config:
      - afi: ipv4
        prefix_lists:
          - name: pl1
            entries:
              - sequence: 3
                action: permit
                prefix: 10.0.0.0/24
      - afi: ipv6
        prefix_lists:
          - name: pl_1
            entries:
              - prefix: '2001:db8:1234::/48'
                action: permit
                sequence: 1
          - name: pl_2
            entries:
              - sequence: 2
                action: remark
                description: TEST_PL1_2
    state: replaced


# Task Output
# -------------
# before:
# - afi: ipv6
#   prefix_lists:
#   - entries:
#     - action: deny
#       prefix: 2001:db8:1234::/48
#       sequence: 1
#     name: pl_1
#   - entries:
#     - action: remark
#       description: TEST_PL_2_REMARK
#       sequence: 2
#     name: pl_2
# - afi: ipv4
#   prefix_lists:
#   - entries:
#     - action: remark
#       description: TEST_PL1_2_REMARK
#       sequence: 3
#     - action: permit
#       prefix: 10.0.0.0/24
#       sequence: 4
#     name: pl1
#   - entries:
#     - action: remark
#       description: TEST_PL2_REMARK
#       sequence: 5
#     name: pl2
# commands:
# - no ipv4 prefix-list pl1 3 remark TEST_PL1_2_REMARK
# - no ipv4 prefix-list pl1 4 permit 10.0.0.0/24
# - ipv4 prefix-list pl1 3 permit 10.0.0.0/24
# - ipv6 prefix-list pl_2 2 remark TEST_PL1_2
# after:
# - afi: ipv6
#   prefix_lists:
#   - entries:
#     - action: deny
#       prefix: 2001:db8:1234::/48
#       sequence: 1
#     name: pl_1
#   - entries:
#     - action: remark
#       description: TEST_PL1_2
#       sequence: 2
#     name: pl_2
# - afi: ipv4
#   prefix_lists:
#   - entries:
#     - action: permit
#       prefix: 10.0.0.0/24
#       sequence: 3
#     name: pl1
#   - entries:
#     - action: remark
#       description: TEST_PL2_REMARK
#       sequence: 5
#     name: pl2


# After state:
# RP/0/0/CPU0:10#show running-config
#
# ipv6 prefix-list pl_1
#  1 deny 2001:db8:1234::/48
# !
# ipv6 prefix-list pl_2
#  2 remark TEST_PL1_2
# !
# ipv4 prefix-list pl1
#  3 permit 10.0.0.0/24
# !
# ipv4 prefix-list pl2
#  5 remark TEST_PL2_REMARK
#
# Module Execution:
#


# Using deleted:


# Before state:
# -------------
# RP/0/0/CPU0:10#show running-config
#
# ipv6 prefix-list pl_1
#  1 deny 2001:db8:1234::/48
# !
# ipv6 prefix-list pl_2
#  2 remark TEST_PL_2_REMARK
# !
# ipv4 prefix-list pl1
#  3 remark TEST_PL1_2_REMARK
#  4 permit 10.0.0.0/24
# !
# ipv4 prefix-list pl2
#  5 remark TEST_PL2_REMARK
# ipv4 prefix-list pl3
#  6 permit 35.0.0.0/8 eq 0

- name: Delete all prefix-lists from the device
  cisco.iosxr.iosxr_prefix_lists:
    state: deleted

# Task Output
# -------------
# before:
# - afi: ipv6
#   prefix_lists:
#   - name: pl_1
#     entries:
#     - prefix: 2001:db8:1234::/48
#       action: deny
#       sequence: 1
#   - name: pl_2
#     entries:
#     - sequence: 2
#       action: remark
#       description: TEST_PL_2_REMARK
# - afi: ipv4
#   prefix_lists:
#   - name: pl1
#     entries:
#     - sequence: 3
#       action: remark
#       description: TEST_PL1_2_REMARK
#     - sequence: 4
#       action: permit
#       prefix: 10.0.0.0/24
#   - name: pl2
#     entries:
#     - sequence: 5
#       action: remark
#       description: TEST_PL2_REMARK
#   - name: pl3
#     entries:
#     - sequence: 6
#       action: permit
#       prefix: 35.0.0.0/8
#       eq: 0
# commands:
# - no ipv6 prefix-list pl_1
# - no ipv6 prefix-list pl_2
# - no ipv4 prefix-list pl1
# - no ipv4 prefix-list pl2
# - no ipv4 prefix-list pl3
# after: []


# After state:
# RP/0/0/CPU0:10#show running-config
#

# using gathered:


# After state:
# ------------
# RP/0/0/CPU0:10#show running-config
# ipv6 prefix-list pl_1
#  1 deny 2001:db8:1234::/48
# !
# ipv6 prefix-list pl_2
#  2 remark TEST_PL_2_REMARK
# !
# ipv4 prefix-list pl1
#  3 remark TEST_PL1_2_REMARK
#  4 permit 10.0.0.0/24
# !
# ipv4 prefix-list pl2
#  5 remark TEST_PL2_REMARK
# !
# ipv4 prefix-list pl3
#  6 permit 35.0.0.0/8 eq 0
# !


- name: Gather ACL interfaces facts using gathered state
  cisco.iosxr.iosxr_prefix_lists:
    state: gathered

# gathered:
# - afi: ipv6
#   prefix_lists:
#   - name: pl_1
#     entries:
#     - prefix: 2001:db8:1234::/48
#       action: deny
#       sequence: 1
#   - name: pl_2
#     entries:
#     - sequence: 2
#       action: remark
#       description: TEST_PL_2_REMARK
# - afi: ipv4
#   prefix_lists:
#   - name: pl1
#     entries:
#     - sequence: 3
#       action: remark
#       description: TEST_PL1_2_REMARK
#     - sequence: 4
#       action: permit
#       prefix: 10.0.0.0/24
#   - name: pl2
#     entries:
#     - sequence: 5
#       action: remark
#       description: TEST_PL2_REMARK
#   - name: pl3
#     entries:
#     - sequence: 6
#       action: permit
#       prefix: 35.0.0.0/8
#       eq: 0


# Using parsed:


# parsed.cfg
# ------------------------------
# ipv6 prefix-list pl_1
#  1 deny 2001:db8:1234::/48
# !
# ipv6 prefix-list pl_2
#  2 remark TEST_PL_2_REMARK
# !
# ipv4 prefix-list pl1
#  3 remark TEST_PL1_2_REMARK
#  4 permit 10.0.0.0/24
# !
# ipv4 prefix-list pl2
#  5 remark TEST_PL2_REMARK


- name: Parse externally provided Prefix_lists config to agnostic model
  cisco.iosxr.iosxr_prefix_lists:
    running_config: '{{ lookup(''file'', ''./fixtures/parsed.cfg'') }}'
    state: parsed


# Task Output
# -------------
# parsed:
# - afi: ipv6
#   prefix_lists:
#   - name: pl_1
#     entries:
#     - prefix: 2001:db8:1234::/48
#       action: deny
#       sequence: 1
#   - name: pl_2
#     entries:
#     - sequence: 2
#       action: remark
#       description: TEST_PL_2_REMARK
# - afi: ipv4
#   prefix_lists:
#   - name: pl1
#     entries:
#     - sequence: 3
#       action: remark
#       description: TEST_PL1_2_REMARK
#     - sequence: 4
#       action: permit
#       prefix: 10.0.0.0/24
#   - name: pl2
#     entries:
#     - sequence: 5
#       action: remark
#       description: TEST_PL2_REMARK
#     - sequence: 6
#       action: permit
#       prefix: 35.0.0.0/8
#       eq: 0


# Using rendered:


- name: Render platform specific commands from task input using rendered state
  register: result
  cisco.iosxr.iosxr_prefix_lists:
    config:
      - afi: ipv6
        prefix_lists:
          - name: pl_1
            entries:
              - prefix: '2001:db8:1234::/48'
                action: deny
                sequence: 1
          - name: pl_2
            entries:
              - sequence: 2
                action: remark
                description: TEST_PL_2_REMARK
      - afi: ipv4
        prefix_lists:
          - name: pl1
            entries:
              - sequence: 3
                action: remark
                description: TEST_PL1_2_REMARK
              - sequence: 4
                action: permit
                prefix: 10.0.0.0/24
          - name: pl2
            entries:
              - sequence: 5
                action: remark
                description: TEST_PL2_REMARK
              - sequence: 6
                action: permit
                prefix: 35.0.0.0/8
                eq: 0
    state: rendered


# Task Output
# -------------
# "rendered": [
#         "ipv6 prefix-list pl_1 1 deny 2001:db8:1234::/48",
#         "ipv6 prefix-list pl_2 2 remark TEST_PL_2_REMARK",
#         "ipv4 prefix-list pl1 3 remark TEST_PL1_2_REMARK",
#         "ipv4 prefix-list pl1 4 permit 10.0.0.0/24",
#         "ipv4 prefix-list pl2 5 remark TEST_PL2_REMARK",
#         "ipv4 prefix-list pl2 6 permit 35.0.0.0/8 eq 0"
#     ]

# Using overridden:


# Before state:
# -------------
# RP/0/0/CPU0:10#show running-config
#
# ipv6 prefix-list pl_1
#  1 deny 2001:db8:1234::/48
# !
# ipv6 prefix-list pl_2
#  2 remark TEST_PL_2_REMARK
# !
# ipv4 prefix-list pl1
#  3 remark TEST_PL1_2_REMARK
#  4 permit 10.0.0.0/24
# !
# ipv4 prefix-list pl2
#  5 remark TEST_PL2_REMARK
#
- name: Overridde all Prefix_lists configuration with provided configuration
  cisco.iosxr.iosxr_prefix_lists:
    config:
      - afi: ipv4
        prefix_lists:
          - name: pl3
            entries:
              - sequence: 3
                action: remark
                description: TEST_PL1_3_REMARK
              - sequence: 4
                action: permit
                prefix: 10.0.0.0/24
              - sequence: 6
                action: permit
                prefix: 35.0.0.0/8
                eq: 0
    state: overridden


# Task Output
# -------------
# before:
# - afi: ipv6
#   prefix_lists:
#   - entries:
#     - action: deny
#       prefix: 2001:db8:1234::/48
#       sequence: 1
#     name: pl_1
#   - entries:
#     - action: remark
#       description: TEST_PL_2_REMARK
#       sequence: 2
#     name: pl_2
# - afi: ipv4
#   prefix_lists:
#   - entries:
#     - action: remark
#       description: TEST_PL1_2_REMARK
#       sequence: 3
#     - action: permit
#       prefix: 10.0.0.0/24
#       sequence: 4
#     name: pl1
#   - entries:
#     - action: remark
#       description: TEST_PL2_REMARK
#       sequence: 5
#     name: pl2
# commands:
# - no ipv6 prefix-list pl_1
# - no ipv6 prefix-list pl_2
# - no ipv4 prefix-list pl1
# - no ipv4 prefix-list pl2
# - ipv4 prefix-list pl3 3 remark TEST_PL1_3_REMARK
# - ipv4 prefix-list pl3 4 permit 10.0.0.0/24
# - ipv4 prefix-list pl3 6 permit 35.0.0.0/8 eq 0
# after:
# - afi: ipv4
#   prefix_lists:
#   - entries:
#     - action: remark
#       description: TEST_PL1_3_REMARK
#       sequence: 3
#     - action: permit
#       prefix: 10.0.0.0/24
#       sequence: 4
#     - action: permit
#       prefix: 35.0.0.0/8
#       sequence: 6
#       eq: 0
#     name: pl3


# After state:
# RP/0/0/CPU0:10#show running-config
#
# ipv4 prefix-list pl3
# 3 remark TEST_PL1_3_REMARK
# 4 permit 10.0.0.0/24
# 6 permit 35.0.0.0/8 eq 0
# !
# !
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - "ipv6 prefix-list pl_1 1 deny 2001:db8:1234::/48"
    - "ipv6 prefix-list pl_2 2 remark TEST_PL_2_REMARK"
    - "ipv4 prefix-list pl1 3 remark TEST_PL1_2_REMARK"
    - "ipv4 prefix-list pl1 4 permit 10.0.0.0/24"
    - "ipv4 prefix-list pl2 5 remark TEST_PL2_REMARK"
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - "ipv6 prefix-list pl_1 1 deny 2001:db8:1234::/48"
    - "ipv6 prefix-list pl_2 2 remark TEST_PL_2_REMARK"
    - "ipv4 prefix-list pl1 3 remark TEST_PL1_2_REMARK"
    - "ipv4 prefix-list pl1 4 permit 10.0.0.0/24"
    - "ipv4 prefix-list pl2 5 remark TEST_PL2_REMARK"
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.prefix_lists.prefix_lists import (
    Prefix_listsArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.config.prefix_lists.prefix_lists import (
    Prefix_lists,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Prefix_listsArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Prefix_lists(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: intent_custom_issue_definitions_info
short_description: Information module for Intent Custom Issue Definitions Info
description:
- This module represents an alias of the module intent_custom_issue_definitions_v1_info
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
    - Id path parameter. Get the custom issue definition for the given custom issue definition Id.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Issues GetTheCustomIssueDefinitionForTheGivenCustomIssueDefinitionIdV1
  description: Complete reference of the GetTheCustomIssueDefinitionForTheGivenCustomIssueDefinitionIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-the-custom-issue-definition-for-the-given-custom-issue-definition-id
notes:
  - SDK Method used are
    issues.Issues.get_the_custom_issue_definition_for_the_given_custom_issue_definition_id_v1,

  - Paths used are
    get /intent/api/v1/customIssueDefinitions/{id},
  - It should be noted that this module is an alias of intent_custom_issue_definitions_v1_info

"""

EXAMPLES = r"""
- name: Get Intent Custom Issue Definitions Info by id
  cisco.dnac.intent_custom_issue_definitions_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "id": "string",
        "name": "string",
        "description": "string",
        "profileId": "string",
        "triggerId": "string",
        "rules": [
          {
            "type": "string",
            "severity": 0,
            "facility": "string",
            "mnemonic": "string",
            "pattern": "string",
            "occurrences": 0,
            "durationInMinutes": 0
          }
        ],
        "isEnabled": true,
        "priority": "string",
        "isDeletable": true,
        "isNotificationEnabled": true,
        "createdTime": 0,
        "lastUpdatedTime": 0
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: auth_token_create_v1
short_description: Resource module for Auth Token Create V1
description:
- Manage operation create of the resource Auth Token Create V1.
- >
   API to obtain an access token, which remains valid for 1 hour. The token obtained using this API is required to be
   set as value to the X-Auth-Token HTTP Header for all API calls to Cisco DNA Center.
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options: {}
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Authentication AuthenticationAPIV1
  description: Complete reference of the AuthenticationAPIV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!authentication-api
notes:
  - SDK Method used are
    authentication.Authentication.authentication_api_v1,

  - Paths used are
    post /dna/system/api/v1/auth/token,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.auth_token_create_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "Token": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: diagnostic_validation_sets_v1_info
short_description: Information module for Diagnostic Validation Sets V1
description:
- Get all Diagnostic Validation Sets V1.
- Get Diagnostic Validation Sets V1 by id.
- Retrieves all the validation sets and optionally the contained validations.
- Retrieves validation details for the given validation set id.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  view:
    description:
    - >
      View query parameter. When the query parameter `view=DETAIL` is passed, all validation sets and associated
      validations will be returned. When the query parameter `view=DEFAULT` is passed, only validation sets
      metadata will be returned.
    type: str
  id:
    description:
    - Id path parameter. Validation set id.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Health and Performance RetrievesAllTheValidationSetsV1
  description: Complete reference of the RetrievesAllTheValidationSetsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!retrieves-all-the-validation-sets
- name: Cisco DNA Center documentation for Health and Performance RetrievesValidationDetailsForAValidationSetV1
  description: Complete reference of the RetrievesValidationDetailsForAValidationSetV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!retrieves-validation-details-for-a-validation-set
notes:
  - SDK Method used are
    health_and_performance.HealthAndPerformance.retrieves_all_the_validation_sets_v1,
    health_and_performance.HealthAndPerformance.retrieves_validation_details_for_a_validation_set_v1,

  - Paths used are
    get /dna/intent/api/v1/diagnosticValidationSets,
    get /dna/intent/api/v1/diagnosticValidationSets/{id},

"""

EXAMPLES = r"""
- name: Get all Diagnostic Validation Sets V1
  cisco.dnac.diagnostic_validation_sets_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    view: string
  register: result

- name: Get Diagnostic Validation Sets V1 by id
  cisco.dnac.diagnostic_validation_sets_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "id": "string",
        "name": "string",
        "description": "string",
        "version": "string",
        "validationGroups": [
          {
            "name": "string",
            "id": "string",
            "description": "string",
            "validations": [
              {
                "id": "string",
                "name": "string"
              }
            ]
          }
        ]
      },
      "version": "string"
    }
"""
